@@ -94,7 +94,8 @@ void CV_ChessboardDetectorBadArgTest::run( int /*start_from */)
     img = cb.clone();
     initArgs();
     pattern_size = Size(2,2);
-    errors += run_test_case( Error::StsOutOfRange, "Invlid pattern size" );
+    errors += run_test_case( Error::StsOutOfRange, "Invalid pattern size" );
+
     pattern_size = cbg.cornersSize();
 
     cb.convertTo(img, CV_32F);
