@@ -1,12 +1,10 @@
 #include "perf_precomp.hpp"
 
-using namespace std;
-using namespace cv;
+namespace opencv_test
+{
 using namespace perf;
-using std::tr1::make_tuple;
-using std::tr1::get;
 
-typedef std::tr1::tuple<Size, MatType, int> Size_SrcDepth_DstChannels_t;
+typedef tuple<Size, MatType, int> Size_SrcDepth_DstChannels_t;
 typedef perf::TestBaseWithParam<Size_SrcDepth_DstChannels_t> Size_SrcDepth_DstChannels;
 
 PERF_TEST_P( Size_SrcDepth_DstChannels, merge,
@@ -22,16 +20,21 @@ PERF_TEST_P( Size_SrcDepth_DstChannels, merge,
     int srcDepth = get<1>(GetParam());
     int dstChannels = get<2>(GetParam());
 
+    int maxValue = 255;
+
     vector<Mat> mv;
     for( int i = 0; i < dstChannels; ++i )
     {
         mv.push_back( Mat(sz, CV_MAKETYPE(srcDepth, 1)) );
-        randu(mv[i], 0, 255);
+        randu(mv[i], 0, maxValue);
     }
 
     Mat dst;
     int runs = (sz.width <= 640) ? 8 : 1;
     TEST_CYCLE_MULTIRUN(runs) merge( (vector<Mat> &)mv, dst );
 
-    SANITY_CHECK(dst, 1e-12);
+    double eps = srcDepth <= CV_32S ? 1e-12 : (FLT_EPSILON * maxValue);
+    SANITY_CHECK(dst, eps);
 }
+
+} // namespace
