@@ -387,7 +387,7 @@ public:
 
                 if( c == '-' )
                 {
-                    assert( ptr[1] == '-' && ptr[2] == '>' );
+                    CV_Assert( ptr[1] == '-' && ptr[2] == '>' );
                     mode = 0;
                     ptr += 3;
                 }
@@ -694,7 +694,7 @@ public:
         else if( *ptr == '!' )
         {
             tag_type = CV_XML_DIRECTIVE_TAG;
-            assert( ptr[1] != '-' || ptr[2] != '-' );
+            CV_Assert( ptr[1] != '-' || ptr[2] != '-' );
             ptr++;
         }
         else
