@@ -6,9 +6,6 @@
 
 #include "opencv2/ts.hpp"
 #include "opencv2/ts/ocl_test.hpp"
-#include "opencv2/core/core_c.h"
-
-#include "opencv2/core/cvdef.h"
 #include "opencv2/core/private.hpp"
 #include "opencv2/core/hal/hal.hpp"
 
