@@ -11,9 +11,7 @@
 #if !defined(GAPI_STANDALONE)
 
 #include <opencv2/gapi/opencv_includes.hpp>
-#include <opencv2/gapi/own/types.hpp>
 #include <opencv2/gapi/own/mat.hpp>
-#include <opencv2/gapi/own/scalar.hpp>
 
 namespace cv
 {
@@ -27,7 +25,7 @@ namespace cv
         return result;
     }
 
-           cv::gapi::own::Mat to_own(Mat&&) = delete;
+    cv::gapi::own::Mat to_own(Mat&&) = delete;
 
     inline cv::gapi::own::Mat to_own(Mat const& m) {
         return (m.dims == 2)
@@ -35,30 +33,18 @@ namespace cv
             :  cv::gapi::own::Mat{to_own<int>(m.size), m.type(), m.data};
     };
 
-
-    inline cv::gapi::own::Scalar to_own(const cv::Scalar& s) { return {s[0], s[1], s[2], s[3]}; };
-
-    inline cv::gapi::own::Size to_own (const Size& s) { return {s.width, s.height}; };
-
-    inline cv::gapi::own::Rect to_own (const Rect& r) { return {r.x, r.y, r.width, r.height}; };
-
-
 namespace gapi
 {
 namespace own
 {
+
     inline cv::Mat to_ocv(Mat const& m) {
         return m.dims.empty()
             ? cv::Mat{m.rows, m.cols, m.type(), m.data, m.step}
             : cv::Mat{m.dims, m.type(), m.data};
     }
-           cv::Mat to_ocv(Mat&&)    = delete;
 
-    inline cv::Scalar to_ocv(const Scalar& s) { return {s[0], s[1], s[2], s[3]}; };
-
-    inline cv::Size to_ocv (const Size& s) { return cv::Size(s.width, s.height); };
-
-    inline cv::Rect to_ocv (const Rect& r) { return cv::Rect(r.x, r.y, r.width, r.height); };
+    cv::Mat to_ocv(Mat&&) = delete;
 
 } // namespace own
 } // namespace gapi
