@@ -127,7 +127,7 @@ template<> struct CompileArgTag<MyCustomTypeNoS11N> {
 } // namespace cv
 
 namespace {
-class MyRMatAdapter : public cv::RMat::Adapter {
+class MyRMatAdapter : public cv::RMat::IAdapter {
     cv::Mat m_mat;
     int m_value;
     std::string m_str;
@@ -569,7 +569,6 @@ TEST_F(S11N_Basic, Test_Bind_RunArgs_MatScalar) {
     v[0] = cv::GRunArg{ mat };
     v[1] = cv::GRunArg{ scalar };
     GRunArgsP output = cv::gapi::bind(v);
-    unsigned int i = 0;
     for (auto it : output)
     {
         using T = cv::GRunArgP;
@@ -591,7 +590,6 @@ TEST_F(S11N_Basic, Test_Bind_RunArgs_MatScalar) {
             GAPI_Assert(false && "This value type is not supported!"); // ...maybe because of STANDALONE mode.
             break;
         }
-        i++;
     }
 }
 
