@@ -9,4 +9,4 @@
 
 #include "test_precomp.hpp"
 
-CV_TEST_MAIN("gapi")
+CV_TEST_MAIN("")
