@@ -399,13 +399,13 @@ CV_IMPL double cvGetWindowProperty(const char* name, int prop_id)
 #endif
 }
 
-cv::Rect cvGetWindowImageRect(const char* name)
+cv::Rect cv::getWindowImageRect(const String& winname)
 {
     CV_TRACE_FUNCTION();
-    CV_Assert(name);
+    CV_Assert(!winname.empty());
 
     {
-        auto window = findWindow_(name);
+        auto window = findWindow_(winname);
         if (window)
         {
             return window->getImageRect();
@@ -416,7 +416,7 @@ cv::Rect cvGetWindowImageRect(const char* name)
     auto backend = getCurrentUIBackend();
     if (backend)
     {
-        CV_LOG_WARNING(NULL, "Can't find window with name: '" << name << "'. Do nothing");
+        CV_LOG_WARNING(NULL, "Can't find window with name: '" << winname << "'. Do nothing");
         CV_NOT_FOUND_DEPRECATION;
     }
     else
@@ -427,13 +427,13 @@ cv::Rect cvGetWindowImageRect(const char* name)
 #else
 
     #if defined (HAVE_QT)
-        return cvGetWindowRect_QT(name);
+        return cvGetWindowRect_QT(winname.c_str());
     #elif defined(HAVE_WIN32UI)
-        return cvGetWindowRect_W32(name);
+        return cvGetWindowRect_W32(winname.c_str());
     #elif defined (HAVE_GTK)
-        return cvGetWindowRect_GTK(name);
+        return cvGetWindowRect_GTK(winname.c_str());
     #elif defined (HAVE_COCOA)
-        return cvGetWindowRect_COCOA(name);
+        return cvGetWindowRect_COCOA(winname.c_str());
     #else
         return Rect(-1, -1, -1, -1);
     #endif
@@ -441,12 +441,6 @@ cv::Rect cvGetWindowImageRect(const char* name)
 #endif
 }
 
-cv::Rect cv::getWindowImageRect(const String& winname)
-{
-    CV_TRACE_FUNCTION();
-    return cvGetWindowImageRect(winname.c_str());
-}
-
 void cv::namedWindow( const String& winname, int flags )
 {
     CV_TRACE_FUNCTION();
@@ -969,6 +963,8 @@ void cv::imshow( const String& winname, InputArray _img )
 {
     CV_TRACE_FUNCTION();
 
+    const Size size = _img.size();
+    CV_Assert(size.width>0 && size.height>0);
     {
         cv::AutoLock lock(cv::getWindowMutex());
         cleanupClosedWindows_();
@@ -1001,9 +997,7 @@ void cv::imshow( const String& winname, InputArray _img )
         }
     }
 
-    const Size size = _img.size();
 #ifndef HAVE_OPENGL
-    CV_Assert(size.width>0 && size.height>0);
     {
         Mat img = _img.getMat();
         CvMat c_img = cvMat(img);
@@ -1011,7 +1005,6 @@ void cv::imshow( const String& winname, InputArray _img )
     }
 #else
     const double useGl = getWindowProperty(winname, WND_PROP_OPENGL);
-    CV_Assert(size.width>0 && size.height>0);
 
     if (useGl <= 0)
     {
