@@ -17,11 +17,7 @@ void cvtYUVtoBGR(const uchar * src_data, size_t src_step,
                  uchar * dst_data, size_t dst_step,
                  int width, int height,
                  int depth, int dcn, bool swapBlue, bool isCbCr);
-void cvtTwoPlaneYUVtoBGR(const uchar * src_data, size_t src_step,
-                         uchar * dst_data, size_t dst_step,
-                         int dst_width, int dst_height,
-                         int dcn, bool swapBlue, int uIdx);
-void cvtTwoPlaneYUVtoBGR(const uchar * y_data, const uchar * uv_data, size_t src_step,
+void cvtTwoPlaneYUVtoBGR(const uchar * y_data, size_t y_step, const uchar * uv_data, size_t uv_step,
                          uchar * dst_data, size_t dst_step,
                          int dst_width, int dst_height,
                          int dcn, bool swapBlue, int uIdx);
@@ -347,16 +343,16 @@ struct RGB2YCrCb_i<ushort>
             sr0 = sr0 - sy0; sr1 = sr1 - sy1;
             sb0 = sb0 - sy0; sb1 = sb1 - sy1;
 
-            v_int32 scr0, scr1, scb0, scb1;
+            v_int32 v_scr0, v_scr1, v_scb0, v_scb1;
 
-            scr0 = (sr0*vc3 + vdd) >> shift;
-            scr1 = (sr1*vc3 + vdd) >> shift;
-            scb0 = (sb0*vc4 + vdd) >> shift;
-            scb1 = (sb1*vc4 + vdd) >> shift;
+            v_scr0 = (sr0*vc3 + vdd) >> shift;
+            v_scr1 = (sr1*vc3 + vdd) >> shift;
+            v_scb0 = (sb0*vc4 + vdd) >> shift;
+            v_scb1 = (sb1*vc4 + vdd) >> shift;
 
             // saturate and pack
-            cr = v_pack_u(scr0, scr1);
-            cb = v_pack_u(scb0, scb1);
+            cr = v_pack_u(v_scr0, v_scr1);
+            cb = v_pack_u(v_scb0, v_scb1);
 
             if(yuvOrder)
             {
@@ -781,36 +777,36 @@ struct YCrCb2RGB_i<uchar>
             v_int8 scr = v_reinterpret_as_s8(cr);
             v_int8 scb = v_reinterpret_as_s8(cb);
 
-            v_int16 scr0, scr1, scb0, scb1;
-            v_expand(scr, scr0, scr1);
-            v_expand(scb, scb0, scb1);
+            v_int16 v_scr0, v_scr1, v_scb0, v_scb1;
+            v_expand(scr, v_scr0, v_scr1);
+            v_expand(scb, v_scb0, v_scb1);
 
             v_int32 b00, b01, b10, b11;
             v_int32 g00, g01, g10, g11;
             v_int32 r00, r01, r10, r11;
 
-            v_mul_expand(scb0, vc3, b00, b01);
-            v_mul_expand(scb1, vc3, b10, b11);
+            v_mul_expand(v_scb0, vc3, b00, b01);
+            v_mul_expand(v_scb1, vc3, b10, b11);
             if(yuvOrder)
             {
                 // if YUV then C3 > 2^15
                 // so we fix the multiplication
                 v_int32 cb00, cb01, cb10, cb11;
-                v_expand(scb0, cb00, cb01);
-                v_expand(scb1, cb10, cb11);
+                v_expand(v_scb0, cb00, cb01);
+                v_expand(v_scb1, cb10, cb11);
                 b00 += cb00 << 15; b01 += cb01 << 15;
                 b10 += cb10 << 15; b11 += cb11 << 15;
             }
 
             v_int32 t00, t01, t10, t11;
-            v_mul_expand(scb0, vc2, t00, t01);
-            v_mul_expand(scb1, vc2, t10, t11);
-            v_mul_expand(scr0, vc1, g00, g01);
-            v_mul_expand(scr1, vc1, g10, g11);
+            v_mul_expand(v_scb0, vc2, t00, t01);
+            v_mul_expand(v_scb1, vc2, t10, t11);
+            v_mul_expand(v_scr0, vc1, g00, g01);
+            v_mul_expand(v_scr1, vc1, g10, g11);
             g00 += t00; g01 += t01;
             g10 += t10; g11 += t11;
-            v_mul_expand(scr0, vc0, r00, r01);
-            v_mul_expand(scr1, vc0, r10, r11);
+            v_mul_expand(v_scr0, vc0, r00, r01);
+            v_mul_expand(v_scr1, vc0, r10, r11);
 
             b00 = (b00 + vdescale) >> shift; b01 = (b01 + vdescale) >> shift;
             b10 = (b10 + vdescale) >> shift; b11 = (b11 + vdescale) >> shift;
@@ -1177,24 +1173,28 @@ struct YUV420sp2RGB8Invoker : ParallelLoopBody
     uchar * dst_data;
     size_t dst_step;
     int width;
-    const uchar* my1, *muv;
-    size_t stride;
+    const uchar* my1;
+    size_t my1_step;
+    const uchar* muv;
+    size_t muv_step;
 
-    YUV420sp2RGB8Invoker(uchar * _dst_data, size_t _dst_step, int _dst_width, size_t _stride, const uchar* _y1, const uchar* _uv)
-        : dst_data(_dst_data), dst_step(_dst_step), width(_dst_width), my1(_y1), muv(_uv), stride(_stride) {}
+    YUV420sp2RGB8Invoker(uchar * _dst_data, size_t _dst_step, int _dst_width,
+                         const uchar* _y1, size_t _y1_step, const uchar* _uv, size_t _uv_step) :
+            dst_data(_dst_data), dst_step(_dst_step), width(_dst_width),
+            my1(_y1), my1_step(_y1_step), muv(_uv), muv_step(_uv_step) {}
 
     void operator()(const Range& range) const CV_OVERRIDE
     {
         const int rangeBegin = range.start * 2;
         const int rangeEnd   = range.end   * 2;
 
-        const uchar* y1 = my1 + rangeBegin * stride, *uv = muv + rangeBegin * stride / 2;
+        const uchar* y1 = my1 + rangeBegin * my1_step, *uv = muv + rangeBegin * muv_step / 2;
 
-        for (int j = rangeBegin; j < rangeEnd; j += 2, y1 += stride * 2, uv += stride)
+        for (int j = rangeBegin; j < rangeEnd; j += 2, y1 += my1_step * 2, uv += muv_step)
         {
             uchar* row1 = dst_data + dst_step * j;
             uchar* row2 = dst_data + dst_step * (j + 1);
-            const uchar* y2 = y1 + stride;
+            const uchar* y2 = y1 + my1_step;
 
             int i = 0;
 #if CV_SIMD
@@ -1395,9 +1395,10 @@ struct YUV420p2RGB8Invoker : ParallelLoopBody
 #define MIN_SIZE_FOR_PARALLEL_YUV420_CONVERSION (320*240)
 
 template<int bIdx, int uIdx, int dcn>
-inline void cvtYUV420sp2RGB(uchar * dst_data, size_t dst_step, int dst_width, int dst_height, size_t _stride, const uchar* _y1, const uchar* _uv)
+inline void cvtYUV420sp2RGB(uchar * dst_data, size_t dst_step, int dst_width, int dst_height,
+                            const uchar* _y1, size_t _y1_step, const uchar* _uv, size_t _uv_step)
 {
-    YUV420sp2RGB8Invoker<bIdx, uIdx, dcn> converter(dst_data, dst_step, dst_width, _stride, _y1,  _uv);
+    YUV420sp2RGB8Invoker<bIdx, uIdx, dcn> converter(dst_data, dst_step, dst_width, _y1, _y1_step, _uv, _uv_step);
     if (dst_width * dst_height >= MIN_SIZE_FOR_PARALLEL_YUV420_CONVERSION)
         parallel_for_(Range(0, dst_height/2), converter);
     else
@@ -1817,26 +1818,16 @@ void cvtYUVtoBGR(const uchar * src_data, size_t src_step,
         CvtColorLoop(src_data, src_step, dst_data, dst_step, width, height, YCrCb2RGB_f<float>(dcn, blueIdx, isCbCr));
 }
 
-void cvtTwoPlaneYUVtoBGR(const uchar * src_data, size_t src_step,
-                         uchar * dst_data, size_t dst_step,
-                         int dst_width, int dst_height,
-                         int dcn, bool swapBlue, int uIdx)
-{
-    CV_INSTRUMENT_REGION();
-
-    const uchar* uv = src_data + src_step * static_cast<size_t>(dst_height);
-    cvtTwoPlaneYUVtoBGR(src_data, uv, src_step, dst_data, dst_step, dst_width, dst_height, dcn, swapBlue, uIdx);
-}
-
 typedef void (*cvt_2plane_yuv_ptr_t)(uchar * /* dst_data*/,
                        size_t /* dst_step */,
                        int /* dst_width */,
                        int /* dst_height */,
-                       size_t /* _stride */,
                        const uchar* /* _y1 */,
-                       const uchar* /* _uv */);
+                       size_t /* _y1_step */,
+                       const uchar* /* _uv */,
+                       size_t /* _uv_step */);
 
-void cvtTwoPlaneYUVtoBGR(const uchar * y_data, const uchar * uv_data, size_t src_step,
+void cvtTwoPlaneYUVtoBGR(const uchar * y_data, size_t y_step, const uchar * uv_data, size_t uv_step,
                          uchar * dst_data, size_t dst_step,
                          int dst_width, int dst_height,
                          int dcn, bool swapBlue, int uIdx)
@@ -1859,7 +1850,7 @@ void cvtTwoPlaneYUVtoBGR(const uchar * y_data, const uchar * uv_data, size_t src
     default: CV_Error( CV_StsBadFlag, "Unknown/unsupported color conversion code" ); break;
     };
 
-    cvtPtr(dst_data, dst_step, dst_width, dst_height, src_step, y_data, uv_data);
+    cvtPtr(dst_data, dst_step, dst_width, dst_height, y_data, y_step, uv_data, uv_step);
 }
 
 typedef void (*cvt_3plane_yuv_ptr_t)(uchar * /* dst_data */,
