@@ -46,7 +46,7 @@ cvMaxRect( const CvRect* rect1, const CvRect* rect2 )
 {
     if( rect1 && rect2 )
     {
-        CvRect max_rect;
+        cv::Rect max_rect;
         int a, b;
 
         max_rect.x = a = rect1->x;
@@ -72,7 +72,7 @@ cvMaxRect( const CvRect* rect1, const CvRect* rect2 )
         if( max_rect.height < b )
             max_rect.height = b;
         max_rect.height -= max_rect.y;
-        return max_rect;
+        return cvRect(max_rect);
     }
     else if( rect1 )
         return *rect1;
@@ -92,97 +92,39 @@ cvBoxPoints( CvBox2D box, CvPoint2D32f pt[4] )
 }
 
 
-int
-icvIntersectLines( double x1, double dx1, double y1, double dy1,
-                   double x2, double dx2, double y2, double dy2, double *t2 )
+double cv::pointPolygonTest( InputArray _contour, Point2f pt, bool measureDist )
 {
-    double d = dx1 * dy2 - dx2 * dy1;
-    int result = -1;
+    CV_INSTRUMENT_REGION();
 
-    if( d != 0 )
-    {
-        *t2 = ((x2 - x1) * dy1 - (y2 - y1) * dx1) / d;
-        result = 0;
-    }
-    return result;
-}
-
-
-void
-icvIntersectLines3( double *a0, double *b0, double *c0,
-                    double *a1, double *b1, double *c1, CvPoint2D32f * point )
-{
-    double det = a0[0] * b1[0] - a1[0] * b0[0];
-
-    if( det != 0 )
-    {
-        det = 1. / det;
-        point->x = (float) ((b0[0] * c1[0] - b1[0] * c0[0]) * det);
-        point->y = (float) ((a1[0] * c0[0] - a0[0] * c1[0]) * det);
-    }
-    else
-    {
-        point->x = point->y = FLT_MAX;
-    }
-}
-
-
-CV_IMPL double
-cvPointPolygonTest( const CvArr* _contour, CvPoint2D32f pt, int measure_dist )
-{
     double result = 0;
+    Mat contour = _contour.getMat();
+    int i, total = contour.checkVector(2), counter = 0;
+    int depth = contour.depth();
+    CV_Assert( total >= 0 && (depth == CV_32S || depth == CV_32F));
 
-    CvSeqBlock block;
-    CvContour header;
-    CvSeq* contour = (CvSeq*)_contour;
-    CvSeqReader reader;
-    int i, total, counter = 0;
-    int is_float;
+    bool is_float = depth == CV_32F;
     double min_dist_num = FLT_MAX, min_dist_denom = 1;
-    CvPoint ip = {0,0};
+    Point ip(cvRound(pt.x), cvRound(pt.y));
 
-    if( !CV_IS_SEQ(contour) )
-    {
-        contour = cvPointSeqFromMat( CV_SEQ_KIND_CURVE + CV_SEQ_FLAG_CLOSED,
-                                    _contour, &header, &block );
-    }
-    else if( CV_IS_SEQ_POINT_SET(contour) )
-    {
-        if( contour->header_size == sizeof(CvContour) && !measure_dist )
-        {
-            CvRect r = ((CvContour*)contour)->rect;
-            if( pt.x < r.x || pt.y < r.y ||
-                pt.x >= r.x + r.width || pt.y >= r.y + r.height )
-                return -1;
-        }
-    }
-    else if( CV_IS_SEQ_CHAIN(contour) )
-    {
-        CV_Error( CV_StsBadArg,
-            "Chains are not supported. Convert them to polygonal representation using cvApproxChains()" );
-    }
-    else
-        CV_Error( CV_StsBadArg, "Input contour is neither a valid sequence nor a matrix" );
+    if( total == 0 )
+        return measureDist ? -DBL_MAX : -1;
 
-    total = contour->total;
-    is_float = CV_SEQ_ELTYPE(contour) == CV_32FC2;
-    cvStartReadSeq( contour, &reader, -1 );
+    const Point* cnt = contour.ptr<Point>();
+    const Point2f* cntf = (const Point2f*)cnt;
 
-    if( !is_float && !measure_dist && (ip.x = cvRound(pt.x)) == pt.x && (ip.y = cvRound(pt.y)) == pt.y )
+    if( !is_float && !measureDist && ip.x == pt.x && ip.y == pt.y )
     {
-        // the fastest "pure integer" branch
-        CvPoint v0, v;
-        CV_READ_SEQ_ELEM( v, reader );
+        // the fastest "purely integer" branch
+        Point v0, v = cnt[total-1];
 
         for( i = 0; i < total; i++ )
         {
-            int dist;
             v0 = v;
-            CV_READ_SEQ_ELEM( v, reader );
+            v = cnt[i];
 
             if( (v0.y <= ip.y && v.y <= ip.y) ||
-                (v0.y > ip.y && v.y > ip.y) ||
-                (v0.x < ip.x && v.x < ip.x) )
+               (v0.y > ip.y && v.y > ip.y) ||
+               (v0.x < ip.x && v.x < ip.x) )
             {
                 if( ip.y == v.y && (ip.x == v.x || (ip.y == v0.y &&
                     ((v0.x <= ip.x && ip.x <= v.x) || (v.x <= ip.x && ip.x <= v0.x)))) )
@@ -190,7 +132,8 @@ cvPointPolygonTest( const CvArr* _contour, CvPoint2D32f pt, int measure_dist )
                 continue;
             }
 
-            dist = (ip.y - v0.y)*(v.x - v0.x) - (ip.x - v0.x)*(v.y - v0.y);
+            int64 dist = static_cast<int64>(ip.y - v0.y)*(v.x - v0.x)
+                       - static_cast<int64>(ip.x - v0.x)*(v.y - v0.y);
             if( dist == 0 )
                 return 0;
             if( v.y < v0.y )
@@ -202,38 +145,32 @@ cvPointPolygonTest( const CvArr* _contour, CvPoint2D32f pt, int measure_dist )
     }
     else
     {
-        CvPoint2D32f v0, v;
-        CvPoint iv;
+        Point2f v0, v;
+        Point iv;
 
         if( is_float )
         {
-            CV_READ_SEQ_ELEM( v, reader );
+            v = cntf[total-1];
         }
         else
         {
-            CV_READ_SEQ_ELEM( iv, reader );
-            v = cvPointTo32f( iv );
+            v = cnt[total-1];
         }
 
-        if( !measure_dist )
+        if( !measureDist )
         {
             for( i = 0; i < total; i++ )
             {
                 double dist;
                 v0 = v;
                 if( is_float )
-                {
-                    CV_READ_SEQ_ELEM( v, reader );
-                }
+                    v = cntf[i];
                 else
-                {
-                    CV_READ_SEQ_ELEM( iv, reader );
-                    v = cvPointTo32f( iv );
-                }
+                    v = cnt[i];
 
                 if( (v0.y <= pt.y && v.y <= pt.y) ||
-                    (v0.y > pt.y && v.y > pt.y) ||
-                    (v0.x < pt.x && v.x < pt.x) )
+                   (v0.y > pt.y && v.y > pt.y) ||
+                   (v0.x < pt.x && v.x < pt.x) )
                 {
                     if( pt.y == v.y && (pt.x == v.x || (pt.y == v0.y &&
                         ((v0.x <= pt.x && pt.x <= v.x) || (v.x <= pt.x && pt.x <= v0.x)))) )
@@ -259,14 +196,9 @@ cvPointPolygonTest( const CvArr* _contour, CvPoint2D32f pt, int measure_dist )
 
                 v0 = v;
                 if( is_float )
-                {
-                    CV_READ_SEQ_ELEM( v, reader );
-                }
+                    v = cntf[i];
                 else
-                {
-                    CV_READ_SEQ_ELEM( iv, reader );
-                    v = cvPointTo32f( iv );
-                }
+                    v = cnt[i];
 
                 dx = v.x - v0.x; dy = v.y - v0.y;
                 dx1 = pt.x - v0.x; dy1 = pt.y - v0.y;
@@ -292,8 +224,8 @@ cvPointPolygonTest( const CvArr* _contour, CvPoint2D32f pt, int measure_dist )
                 }
 
                 if( (v0.y <= pt.y && v.y <= pt.y) ||
-                    (v0.y > pt.y && v.y > pt.y) ||
-                    (v0.x < pt.x && v.x < pt.x) )
+                   (v0.y > pt.y && v.y > pt.y) ||
+                   (v0.x < pt.x && v.x < pt.x) )
                     continue;
 
                 dist_num = dy1*dx - dx1*dy;
@@ -302,7 +234,7 @@ cvPointPolygonTest( const CvArr* _contour, CvPoint2D32f pt, int measure_dist )
                 counter += dist_num > 0;
             }
 
-            result = sqrt(min_dist_num/min_dist_denom);
+            result = std::sqrt(min_dist_num/min_dist_denom);
             if( counter % 2 == 0 )
                 result = -result;
         }
@@ -312,6 +244,14 @@ cvPointPolygonTest( const CvArr* _contour, CvPoint2D32f pt, int measure_dist )
 }
 
 
+CV_IMPL double
+cvPointPolygonTest( const CvArr* _contour, CvPoint2D32f pt, int measure_dist )
+{
+    cv::AutoBuffer<double> abuf;
+    cv::Mat contour = cv::cvarrToMat(_contour, false, false, 0, &abuf);
+    return cv::pointPolygonTest(contour, pt, measure_dist != 0);
+}
+
 /*
  This code is described in "Computational Geometry in C" (Second Edition),
  Chapter 7.  It is not written to be comprehensible without the
@@ -339,7 +279,7 @@ static int areaSign( Point2f a, Point2f b, Point2f c )
 }
 
 //---------------------------------------------------------------------
-// Returns true iff point c lies on the closed segement ab.
+// Returns true iff point c lies on the closed segment ab.
 // Assumes it is already known that abc are collinear.
 //---------------------------------------------------------------------
 static bool between( Point2f a, Point2f b, Point2f c )
@@ -355,11 +295,19 @@ static bool between( Point2f a, Point2f b, Point2f c )
         ((a.y >= c.y) && (c.y >= b.y));
 }
 
-static char parallelInt( Point2f a, Point2f b, Point2f c, Point2f d, Point2f& p, Point2f& q )
+enum LineSegmentIntersection
 {
-    char code = 'e';
+    LS_NO_INTERSECTION = 0,
+    LS_SINGLE_INTERSECTION = 1,
+    LS_OVERLAP = 2,
+    LS_ENDPOINT_INTERSECTION = 3
+};
+
+static LineSegmentIntersection parallelInt( Point2f a, Point2f b, Point2f c, Point2f d, Point2f& p, Point2f& q )
+{
+    LineSegmentIntersection code = LS_OVERLAP;
     if( areaSign(a, b, c) != 0 )
-        code = '0';
+        code = LS_NO_INTERSECTION;
     else if( between(a, b, c) && between(a, b, d))
         p = c, q = d;
     else if( between(c, d, a) && between(c, d, b))
@@ -373,60 +321,33 @@ static char parallelInt( Point2f a, Point2f b, Point2f c, Point2f d, Point2f& p,
     else if( between(a, b, d) && between(c, d, a))
         p = d, q = a;
     else
-        code = '0';
+        code = LS_NO_INTERSECTION;
     return code;
 }
 
-//---------------------------------------------------------------------
-// segSegInt: Finds the point of intersection p between two closed
-// segments ab and cd.  Returns p and a char with the following meaning:
-// 'e': The segments collinearly overlap, sharing a point.
-// 'v': An endpoint (vertex) of one segment is on the other segment,
-// but 'e' doesn't hold.
-// '1': The segments intersect properly (i.e., they share a point and
-// neither 'v' nor 'e' holds).
-// '0': The segments do not intersect (i.e., they share no points).
-// Note that two collinear segments that share just one point, an endpoint
-// of each, returns 'e' rather than 'v' as one might expect.
-//---------------------------------------------------------------------
-static char segSegInt( Point2f a, Point2f b, Point2f c, Point2f d, Point2f& p, Point2f& q )
+// Finds intersection of two line segments: (a, b) and (c, d).
+static LineSegmentIntersection intersectLineSegments( Point2f a, Point2f b, Point2f c,
+                                                      Point2f d, Point2f& p, Point2f& q )
 {
-    double  s, t;       // The two parameters of the parametric eqns.
-    double num, denom;  // Numerator and denoninator of equations.
-    char code = '?';    // Return char characterizing intersection.
-
-    denom = a.x * (double)( d.y - c.y ) +
-    b.x * (double)( c.y - d.y ) +
-    d.x * (double)( b.y - a.y ) +
-    c.x * (double)( a.y - b.y );
+    double denom = a.x * (double)(d.y - c.y) + b.x * (double)(c.y - d.y) +
+                   d.x * (double)(b.y - a.y) + c.x * (double)(a.y - b.y);
 
     // If denom is zero, then segments are parallel: handle separately.
-    if (denom == 0.0)
+    if( denom == 0. )
         return parallelInt(a, b, c, d, p, q);
 
-    num =    a.x * (double)( d.y - c.y ) +
-    c.x * (double)( a.y - d.y ) +
-    d.x * (double)( c.y - a.y );
-    if ( (num == 0.0) || (num == denom) ) code = 'v';
-    s = num / denom;
+    double num = a.x * (double)(d.y - c.y) + c.x * (double)(a.y - d.y) + d.x * (double)(c.y - a.y);
+    double s = num / denom;
 
-    num = -( a.x * (double)( c.y - b.y ) +
-            b.x * (double)( a.y - c.y ) +
-            c.x * (double)( b.y - a.y ) );
-    if ( (num == 0.0) || (num == denom) ) code = 'v';
-    t = num / denom;
-
-    if      ( (0.0 < s) && (s < 1.0) &&
-             (0.0 < t) && (t < 1.0) )
-        code = '1';
-    else if ( (0.0 > s) || (s > 1.0) ||
-             (0.0 > t) || (t > 1.0) )
-        code = '0';
+    num = a.x * (double)(b.y - c.y) + b.x * (double)(c.y - a.y) + c.x * (double)(a.y - b.y);
+    double t = num / denom;
 
     p.x = (float)(a.x + s*(b.x - a.x));
     p.y = (float)(a.y + s*(b.y - a.y));
+    q = p;
 
-    return code;
+    return s < 0. || s > 1. || t < 0. || t > 1. ? LS_NO_INTERSECTION :
+           s == 0. || s == 1. || t == 0. || t == 1. ? LS_ENDPOINT_INTERSECTION : LS_SINGLE_INTERSECTION;
 }
 
 static tInFlag inOut( Point2f p, tInFlag inflag, int aHB, int bHA, Point2f*& result )
@@ -484,8 +405,8 @@ static int intersectConvexConvex_( const Point2f* P, int n, const Point2f* Q, in
 
         // If A & B intersect, update inflag.
         Point2f p, q;
-        int code = segSegInt( P[a1], P[a], Q[b1], Q[b], p, q );
-        if( code == '1' || code == 'v' )
+        LineSegmentIntersection code = intersectLineSegments( P[a1], P[a], Q[b1], Q[b], p, q );
+        if( code == LS_SINGLE_INTERSECTION || code == LS_ENDPOINT_INTERSECTION )
         {
             if( inflag == Unknown && FirstPoint )
             {
@@ -500,7 +421,7 @@ static int intersectConvexConvex_( const Point2f* P, int n, const Point2f* Q, in
         //-----Advance rules-----
 
         // Special case: A & B overlap and oppositely oriented.
-        if( code == 'e' && A.ddot(B) < 0 )
+        if( code == LS_OVERLAP && A.ddot(B) < 0 )
         {
             addSharedSeg( p, q, result );
             return (int)(result - result0);
@@ -566,6 +487,8 @@ static int intersectConvexConvex_( const Point2f* P, int n, const Point2f* Q, in
 
 float cv::intersectConvexConvex( InputArray _p1, InputArray _p2, OutputArray _p12, bool handleNested )
 {
+    CV_INSTRUMENT_REGION();
+
     Mat p1 = _p1.getMat(), p2 = _p2.getMat();
     CV_Assert( p1.depth() == CV_32S || p1.depth() == CV_32F );
     CV_Assert( p2.depth() == CV_32S || p2.depth() == CV_32F );
@@ -582,7 +505,7 @@ float cv::intersectConvexConvex( InputArray _p1, InputArray _p2, OutputArray _p1
     }
 
     AutoBuffer<Point2f> _result(n*2 + m*2 + 1);
-    Point2f *fp1 = _result, *fp2 = fp1 + n;
+    Point2f *fp1 = _result.data(), *fp2 = fp1 + n;
     Point2f* result = fp2 + m;
     int orientation = 0;
 
@@ -621,21 +544,41 @@ float cv::intersectConvexConvex( InputArray _p1, InputArray _p2, OutputArray _p1
             return 0.f;
         }
 
-        if( pointPolygonTest(_InputArray(fp1, n), fp2[0], false) >= 0 )
+        bool intersected = false;
+
+        // check if all of fp2's vertices is inside/on the edge of fp1.
+        int nVertices = 0;
+        for (int i=0; i<m; ++i)
+            nVertices += pointPolygonTest(_InputArray(fp1, n), fp2[i], false) >= 0;
+
+        // if all of fp2's vertices is inside/on the edge of fp1.
+        if (nVertices == m)
         {
+            intersected = true;
             result = fp2;
             nr = m;
         }
-        else if( pointPolygonTest(_InputArray(fp2, n), fp1[0], false) >= 0 )
+        else // otherwise check if fp2 is inside fp1.
         {
-            result = fp1;
-            nr = n;
+            nVertices = 0;
+            for (int i=0; i<n; ++i)
+                nVertices += pointPolygonTest(_InputArray(fp2, m), fp1[i], false) >= 0;
+
+            // // if all of fp1's vertices is inside/on the edge of fp2.
+            if (nVertices == n)
+            {
+                intersected = true;
+                result = fp1;
+                nr = n;
+            }
         }
-        else
+
+        if (!intersected)
         {
             _p12.release();
             return 0.f;
         }
+
         area = (float)contourArea(_InputArray(result, nr), false);
     }
 
@@ -651,124 +594,3 @@ float cv::intersectConvexConvex( InputArray _p1, InputArray _p2, OutputArray _p1
     }
     return (float)fabs(area);
 }
-
-/*
-static void testConvConv()
-{
-    static const int P1[] =
-    {
-        0,	0,
-        100, 0,
-        100, 100,
-        0, 100,
-    };
-
-    static const int Q1[] =
-    {
-        100, 80,
-        50, 80,
-        50, 50,
-        100, 50
-    };
-
-    static const int P2[] =
-    {
-        0, 0,
-        200, 0,
-        200, 100,
-        100, 200,
-        0, 100
-    };
-
-    static const int Q2[] =
-    {
-        100, 100,
-        300, 100,
-        300, 200,
-        100, 200
-    };
-
-    static const int P3[] =
-    {
-        0,   0,
-        100, 0,
-        100, 100,
-        0, 100
-    };
-
-    static const int Q3[] =
-    {
-        50, 50,
-        150, 50,
-        150, 150,
-        50, 150
-    };
-
-    static const int P4[] =
-    {
-        0, 160,
-        50, 80,
-        130, 0,
-        190, 20,
-        240, 100,
-        240, 260,
-        190, 290,
-        130, 320,
-        70, 320,
-        30, 290
-    };
-
-    static const int Q4[] =
-    {
-        160, -30,
-        280, 160,
-        160, 320,
-        0, 220,
-        30, 100
-    };
-
-    static const void* PQs[] =
-    {
-        P1, Q1, P2, Q2, P3, Q3, P4, Q4
-    };
-
-    static const int lens[] =
-    {
-        CV_DIM(P1), CV_DIM(Q1),
-        CV_DIM(P2), CV_DIM(Q2),
-        CV_DIM(P3), CV_DIM(Q3),
-        CV_DIM(P4), CV_DIM(Q4)
-    };
-
-    Mat img(800, 800, CV_8UC3);
-
-    for( int i = 0; i < CV_DIM(PQs)/2; i++ )
-    {
-        Mat Pm = Mat(lens[i*2]/2, 1, CV_32SC2, (void*)PQs[i*2]) + Scalar(100, 100);
-        Mat Qm = Mat(lens[i*2+1]/2, 1, CV_32SC2, (void*)PQs[i*2+1]) + Scalar(100, 100);
-        Point* P = Pm.ptr<Point>();
-        Point* Q = Qm.ptr<Point>();
-
-        flip(Pm, Pm, 0);
-        flip(Qm, Qm, 0);
-
-        Mat Rm;
-        intersectConvexConvex(Pm, Qm, Rm);
-        std::cout << Rm << std::endl << std::endl;
-
-        img = Scalar::all(0);
-
-        polylines(img, Pm, true, Scalar(0,255,0), 1, CV_AA, 0);
-        polylines(img, Qm, true, Scalar(0,0,255), 1, CV_AA, 0);
-        Mat temp;
-        Rm.convertTo(temp, CV_32S, 256);
-        polylines(img, temp, true, Scalar(128, 255, 255), 3, CV_AA, 8);
-
-        namedWindow("test", 1);
-        imshow("test", img);
-        waitKey();
-    }
-}
-*/
-
-/* End of file. */
