@@ -87,7 +87,7 @@ public:
     {
         return ffmpegCapture ? icvGrabFrame_FFMPEG_p(ffmpegCapture)!=0 : false;
     }
-    virtual bool retrieveFrame(int, cv::OutputArray frame) CV_OVERRIDE
+    virtual bool retrieveFrame(int flag, cv::OutputArray frame) CV_OVERRIDE
     {
         unsigned char* data = 0;
         int step=0, width=0, height=0, cn=0;
@@ -102,8 +102,14 @@ public:
             }
         }
 
-        if (!icvRetrieveFrame_FFMPEG_p(ffmpegCapture, &data, &step, &width, &height, &cn))
-            return false;
+        if (flag == 0) {
+            if (!icvRetrieveFrame_FFMPEG_p(ffmpegCapture, &data, &step, &width, &height, &cn))
+                return false;
+        }
+        else {
+            if (!ffmpegCapture->retrieveFrame(flag, &data, &step, &width, &height, &cn))
+                return false;
+        }
 
         cv::Mat tmp(height, width, CV_MAKETYPE(CV_8U, cn), data, step);
         this->rotateFrame(tmp);
