@@ -9,17 +9,39 @@
 #include "opencv2/ts.hpp"
 #include "opencv2/videoio.hpp"
 #include "opencv2/videoio/registry.hpp"
-#include "opencv2/imgproc/imgproc_c.h"
-
 #include "opencv2/core/private.hpp"
 
 namespace cv {
 
-inline std::ostream &operator<<(std::ostream &out, const VideoCaptureAPIs& api)
+static inline
+std::ostream& operator<<(std::ostream& out, const VideoCaptureAPIs& api)
 {
     out << cv::videoio_registry::getBackendName(api); return out;
 }
 
+static inline
+std::ostream& operator<<(std::ostream& out, const VideoAccelerationType& va_type)
+{
+    struct {
+        VideoAccelerationType va_type;
+        const char* str;
+    } va_types[] = {
+            {VIDEO_ACCELERATION_ANY,   "ANY"},
+            {VIDEO_ACCELERATION_NONE,  "NONE"},
+            {VIDEO_ACCELERATION_D3D11, "D3D11"},
+            {VIDEO_ACCELERATION_VAAPI, "VAAPI"},
+            {VIDEO_ACCELERATION_MFX,   "MFX"},
+    };
+    for (const auto& va : va_types) {
+        if (va_type == va.va_type) {
+            out << va.str;
+            return out;
+        }
+    }
+    out << cv::format("UNKNOWN(0x%ux)", static_cast<unsigned int>(va_type));
+    return out;
+}
+
 static inline void PrintTo(const cv::VideoCaptureAPIs& api, std::ostream* os)
 {
     *os << cv::videoio_registry::getBackendName(api);
