@@ -1,47 +1,8 @@
-/*M///////////////////////////////////////////////////////////////////////////////////////
-//
-//  IMPORTANT: READ BEFORE DOWNLOADING, COPYING, INSTALLING OR USING.
-//
-//  By downloading, copying, installing or using the software you agree to this license.
-//  If you do not agree to this license, do not download, install,
-//  copy or use the software.
-//
-//
-//                           License Agreement
-//                For Open Source Computer Vision Library
-//
-// Copyright (C) 2000-2008, Intel Corporation, all rights reserved.
-// Copyright (C) 2009, Willow Garage Inc., all rights reserved.
-// Third party copyrights are property of their respective owners.
-//
-// Redistribution and use in source and binary forms, with or without modification,
-// are permitted provided that the following conditions are met:
-//
-//   * Redistribution's of source code must retain the above copyright notice,
-//     this list of conditions and the following disclaimer.
-//
-//   * Redistribution's in binary form must reproduce the above copyright notice,
-//     this list of conditions and the following disclaimer in the documentation
-//     and/or other materials provided with the distribution.
-//
-//   * The name of the copyright holders may not be used to endorse or promote products
-//     derived from this software without specific prior written permission.
-//
-// This software is provided by the copyright holders and contributors "as is" and
-// any express or implied warranties, including, but not limited to, the implied
-// warranties of merchantability and fitness for a particular purpose are disclaimed.
-// In no event shall the Intel Corporation or contributors be liable for any direct,
-// indirect, incidental, special, exemplary, or consequential damages
-// (including, but not limited to, procurement of substitute goods or services;
-// loss of use, data, or profits; or business interruption) however caused
-// and on any theory of liability, whether in contract, strict liability,
-// or tort (including negligence or otherwise) arising in any way out of
-// the use of this software, even if advised of the possibility of such damage.
-//
-//M*/
+// This file is part of OpenCV project.
+// It is subject to the license terms in the LICENSE file found in the top-level directory
+// of this distribution and at http://opencv.org/license.html.
 
 #include "test_precomp.hpp"
-#include "opencv2/videoio/videoio_c.h"
 
 namespace opencv_test
 {
@@ -263,7 +224,7 @@ public:
             if (cvtest::debugLevel > 0)
                 std::cout << "i = " << i << ": timestamp = " << timestamp << std::endl;
             const double frame_period = 1000.f/bunny_param.getFps();
-            // NOTE: eps == frame_period, because videoCapture returns frame begining timestamp or frame end
+            // NOTE: eps == frame_period, because videoCapture returns frame beginning timestamp or frame end
             // timestamp depending on codec and back-end. So the first frame has timestamp 0 or frame_period.
             EXPECT_NEAR(timestamp, i*frame_period, frame_period) << "i=" << i;
         }
@@ -431,11 +392,11 @@ static Ext_Fourcc_PSNR synthetic_params[] = {
     {"mkv", "MPEG", 30.f, CAP_FFMPEG},
     {"mkv", "MJPG", 30.f, CAP_FFMPEG},
 
-    {"avi", "MPEG", 30.f, CAP_GSTREAMER},
+    {"avi", "MPEG", 28.f, CAP_GSTREAMER},
     {"avi", "MJPG", 30.f, CAP_GSTREAMER},
     {"avi", "H264", 30.f, CAP_GSTREAMER},
 
-    {"mkv", "MPEG", 30.f, CAP_GSTREAMER},
+    {"mkv", "MPEG", 28.f, CAP_GSTREAMER},
     {"mkv", "MJPG", 30.f, CAP_GSTREAMER},
     {"mkv", "H264", 30.f, CAP_GSTREAMER},
 
@@ -616,4 +577,409 @@ static vector<Ext_Fourcc_API> generate_Ext_Fourcc_API_nocrash()
 
 INSTANTIATE_TEST_CASE_P(videoio, Videoio_Writer_bad_fourcc, testing::ValuesIn(generate_Ext_Fourcc_API_nocrash()));
 
+typedef testing::TestWithParam<VideoCaptureAPIs> safe_capture;
+
+TEST_P(safe_capture, frames_independency)
+{
+    VideoCaptureAPIs apiPref = GetParam();
+    if (!videoio_registry::hasBackend(apiPref))
+        throw SkipTestException(cv::String("Backend is not available/disabled: ") + cv::videoio_registry::getBackendName(apiPref));
+
+    VideoCapture cap;
+    String video_file = BunnyParameters::getFilename(String(".avi"));
+    EXPECT_NO_THROW(cap.open(video_file, apiPref));
+    if (!cap.isOpened())
+    {
+        std::cout << "SKIP test: backend " << apiPref << " can't open the video: " << video_file << std::endl;
+        return;
+    }
+
+    Mat frames[10];
+    Mat hardCopies[10];
+    for(int i = 0; i < 10; i++)
+    {
+        ASSERT_NO_THROW(cap >> frames[i]);
+        EXPECT_FALSE(frames[i].empty());
+        hardCopies[i] = frames[i].clone();
+    }
+
+    for(int i = 0; i < 10; i++)
+        EXPECT_EQ(0, cv::norm(frames[i], hardCopies[i], NORM_INF)) << i;
+}
+
+static VideoCaptureAPIs safe_apis[] = {CAP_FFMPEG, CAP_GSTREAMER, CAP_MSMF,CAP_AVFOUNDATION};
+INSTANTIATE_TEST_CASE_P(videoio, safe_capture, testing::ValuesIn(safe_apis));
+
+//==================================================================================================
+// TEST_P(videocapture_acceleration, ...)
+
+struct VideoCaptureAccelerationInput
+{
+    const char* filename;
+    double psnr_threshold;
+};
+
+static inline
+std::ostream& operator<<(std::ostream& out, const VideoCaptureAccelerationInput& p)
+{
+    out << p.filename;
+    return out;
+}
+
+typedef testing::TestWithParam<tuple<VideoCaptureAccelerationInput, VideoCaptureAPIs, VideoAccelerationType, bool>> videocapture_acceleration;
+
+TEST_P(videocapture_acceleration, read)
+{
+    auto param = GetParam();
+    std::string filename = get<0>(param).filename;
+    double psnr_threshold = get<0>(param).psnr_threshold;
+    VideoCaptureAPIs backend = get<1>(param);
+    VideoAccelerationType va_type = get<2>(param);
+    bool use_umat = get<3>(param);
+    const int frameNum = 15;
+
+    std::string filepath = cvtest::findDataFile("video/" + filename);
+
+    if (backend == CAP_MSMF && (
+        filename == "sample_322x242_15frames.yuv420p.mjpeg.mp4" ||
+        filename == "sample_322x242_15frames.yuv420p.libx265.mp4" ||
+        filename == "sample_322x242_15frames.yuv420p.libaom-av1.mp4" ||
+        filename == "sample_322x242_15frames.yuv420p.mpeg2video.mp4"
+    ))
+        throw SkipTestException("Format/codec is not supported");
+
+
+    std::string backend_name = cv::videoio_registry::getBackendName(backend);
+    if (!videoio_registry::hasBackend(backend))
+        throw SkipTestException(cv::String("Backend is not available/disabled: ") + backend_name);
+
+
+    // HW reader
+    std::vector<int> params = { CAP_PROP_HW_ACCELERATION, static_cast<int>(va_type) };
+    if (use_umat)
+    {
+        if (backend != CAP_FFMPEG)
+            throw SkipTestException(cv::String("UMat/OpenCL mapping is not supported by current backend: ") + backend_name);
+        if (!cv::videoio_registry::isBackendBuiltIn(backend))
+            throw SkipTestException(cv::String("UMat/OpenCL mapping is not supported through plugins yet: ") + backend_name);
+        params.push_back(CAP_PROP_HW_ACCELERATION_USE_OPENCL);
+        params.push_back(1);
+    }
+    VideoCapture hw_reader(filepath, backend, params);
+    if (!hw_reader.isOpened())
+    {
+        if (use_umat)
+        {
+            throw SkipTestException(backend_name + " VideoCapture on " + filename + " not supported with HW acceleration + OpenCL/Umat mapping, skipping");
+        }
+        else if (va_type == VIDEO_ACCELERATION_ANY || va_type == VIDEO_ACCELERATION_NONE)
+        {
+            // ANY HW acceleration should have fallback to SW codecs
+            VideoCapture sw_reader(filepath, backend, {
+                    CAP_PROP_HW_ACCELERATION, VIDEO_ACCELERATION_NONE
+            });
+            if (!sw_reader.isOpened())
+                throw SkipTestException(backend_name + " VideoCapture on " + filename + " not supported, skipping");
+
+            ASSERT_TRUE(hw_reader.isOpened()) << "ANY HW acceleration should have fallback to SW codecs";
+        }
+        else
+        {
+            throw SkipTestException(backend_name + " VideoCapture on " + filename + " not supported with HW acceleration, skipping");
+        }
+    }
+
+    VideoAccelerationType actual_va = static_cast<VideoAccelerationType>(static_cast<int>(hw_reader.get(CAP_PROP_HW_ACCELERATION)));
+    if (va_type != VIDEO_ACCELERATION_ANY && va_type != VIDEO_ACCELERATION_NONE)
+    {
+        ASSERT_EQ((int)actual_va, (int)va_type) << "actual_va=" << actual_va << ", va_type=" << va_type;
+    }
+    std::cout << "VideoCapture " << backend_name << ":" << actual_va << std::endl << std::flush;
+
+    double min_psnr_original = 1000;
+    for (int i = 0; i < frameNum; i++)
+    {
+        SCOPED_TRACE(cv::format("frame=%d", i));
+        Mat frame;
+        if (use_umat)
+        {
+            UMat umat;
+            bool read_umat_result = hw_reader.read(umat);
+            if (!read_umat_result && i == 0)
+            {
+                if (filename == "sample_322x242_15frames.yuv420p.libvpx-vp9.mp4")
+                    throw SkipTestException("Unable to read the first frame with VP9 codec (media stack misconfiguration / bug)");
+                // FFMPEG: [av1 @ 0000027ac07d1340] Your platform doesn't suppport hardware accelerated AV1 decoding.
+                if (filename == "sample_322x242_15frames.yuv420p.libaom-av1.mp4")
+                    throw SkipTestException("Unable to read the first frame with AV1 codec (missing support)");
+            }
+#ifdef _WIN32
+            if (!read_umat_result && i == 1)
+            {
+                if (filename == "sample_322x242_15frames.yuv420p.libvpx-vp9.mp4")
+                    throw SkipTestException("Unable to read the second frame with VP9 codec (media stack misconfiguration / outdated MSMF version)");
+            }
+#endif
+            EXPECT_TRUE(read_umat_result);
+            ASSERT_FALSE(umat.empty());
+            umat.copyTo(frame);
+        }
+        else
+        {
+            bool read_result = hw_reader.read(frame);
+            if (!read_result && i == 0)
+            {
+                if (filename == "sample_322x242_15frames.yuv420p.libvpx-vp9.mp4")
+                    throw SkipTestException("Unable to read the first frame with VP9 codec (media stack misconfiguration / bug)");
+                // FFMPEG: [av1 @ 0000027ac07d1340] Your platform doesn't suppport hardware accelerated AV1 decoding.
+                if (filename == "sample_322x242_15frames.yuv420p.libaom-av1.mp4")
+                    throw SkipTestException("Unable to read the first frame with AV1 codec (missing support)");
+            }
+#ifdef _WIN32
+            if (!read_result && i == 1)
+            {
+                if (filename == "sample_322x242_15frames.yuv420p.libvpx-vp9.mp4")
+                    throw SkipTestException("Unable to read the second frame with VP9 codec (media stack misconfiguration / outdated MSMF version)");
+            }
+#endif
+            EXPECT_TRUE(read_result);
+        }
+        ASSERT_FALSE(frame.empty());
+
+        if (cvtest::debugLevel > 0)
+        {
+            imwrite(cv::format("test_frame%03d.png", i), frame);
+        }
+
+        Mat original(frame.size(), CV_8UC3, Scalar::all(0));
+        generateFrame(i, frameNum, original);
+        double psnr = cvtest::PSNR(frame, original);
+        if (psnr < min_psnr_original)
+            min_psnr_original = psnr;
+    }
+
+    std::ostringstream ss; ss << actual_va;
+    std::string actual_va_str = ss.str();
+    std::cout << "VideoCapture with acceleration = " << cv::format("%-6s @ %-10s", actual_va_str.c_str(), backend_name.c_str())
+            << " on " << filename
+            << " with PSNR-original = " << min_psnr_original
+            << std::endl << std::flush;
+    EXPECT_GE(min_psnr_original, psnr_threshold);
+}
+
+static const VideoCaptureAccelerationInput hw_filename[] = {
+        { "sample_322x242_15frames.yuv420p.libxvid.mp4", 28.0 },
+        { "sample_322x242_15frames.yuv420p.mjpeg.mp4", 20.0 },
+        { "sample_322x242_15frames.yuv420p.mpeg2video.mp4", 24.0 },  // GSTREAMER on Ubuntu 18.04
+        { "sample_322x242_15frames.yuv420p.libx264.mp4", 24.0 },  // GSTREAMER on Ubuntu 18.04
+        { "sample_322x242_15frames.yuv420p.libx265.mp4", 30.0 },
+        { "sample_322x242_15frames.yuv420p.libvpx-vp9.mp4", 30.0 },
+        { "sample_322x242_15frames.yuv420p.libaom-av1.mp4", 30.0 }
+};
+
+static const VideoCaptureAPIs hw_backends[] = {
+        CAP_FFMPEG,
+        CAP_GSTREAMER,
+#ifdef _WIN32
+        CAP_MSMF,
+#endif
+};
+
+static const VideoAccelerationType hw_types[] = {
+        VIDEO_ACCELERATION_NONE,
+        VIDEO_ACCELERATION_ANY,
+        VIDEO_ACCELERATION_MFX,
+#ifdef _WIN32
+        VIDEO_ACCELERATION_D3D11,
+#else
+        VIDEO_ACCELERATION_VAAPI,
+#endif
+};
+
+static bool hw_use_umat[] = {
+        false,
+        true
+};
+
+INSTANTIATE_TEST_CASE_P(videoio, videocapture_acceleration, testing::Combine(
+    testing::ValuesIn(hw_filename),
+    testing::ValuesIn(hw_backends),
+    testing::ValuesIn(hw_types),
+    testing::ValuesIn(hw_use_umat)
+));
+
+////////////////////////////////////////// TEST_P(video_acceleration, write_read)
+
+typedef tuple<Ext_Fourcc_PSNR, VideoAccelerationType, bool> VATestParams;
+
+typedef testing::TestWithParam<VATestParams> videowriter_acceleration;
+
+TEST_P(videowriter_acceleration, write)
+{
+    auto param = GetParam();
+    VideoCaptureAPIs backend = get<0>(param).api;
+    std::string codecid = get<0>(param).fourcc;
+    std::string extension = get<0>(param).ext;
+    double psnr_threshold = get<0>(param).PSNR;
+    VideoAccelerationType va_type = get<1>(param);
+    bool use_umat = get<2>(param);
+    std::string backend_name = cv::videoio_registry::getBackendName(backend);
+    if (!videoio_registry::hasBackend(backend))
+        throw SkipTestException(cv::String("Backend is not available/disabled: ") + backend_name);
+
+    const Size sz(640, 480);
+    const int frameNum = 15;
+    const double fps = 25;
+
+    std::string filename = tempfile("videowriter_acceleration.") + extension;
+
+    // Write video
+    VideoAccelerationType actual_va;
+    {
+        std::vector<int> params = { VIDEOWRITER_PROP_HW_ACCELERATION, static_cast<int>(va_type) };
+        if (use_umat) {
+            if (backend != CAP_FFMPEG)
+                throw SkipTestException(cv::String("UMat/OpenCL mapping is not supported by current backend: ") + backend_name);
+            if (!cv::videoio_registry::isBackendBuiltIn(backend))
+                throw SkipTestException(cv::String("UMat/OpenCL mapping is not supported through plugins yet: ") + backend_name);
+            params.push_back(VIDEOWRITER_PROP_HW_ACCELERATION_USE_OPENCL);
+            params.push_back(1);
+        }
+        VideoWriter hw_writer(
+            filename,
+            backend,
+            VideoWriter::fourcc(codecid[0], codecid[1], codecid[2], codecid[3]),
+            fps,
+            sz,
+            params
+        );
+
+        if (!hw_writer.isOpened())
+        {
+            if (use_umat)
+            {
+                throw SkipTestException(backend_name + " VideoWriter on " + filename + " not supported with HW acceleration + OpenCL/Umat mapping, skipping");
+            }
+            else if (va_type == VIDEO_ACCELERATION_ANY || va_type == VIDEO_ACCELERATION_NONE)
+            {
+                // ANY HW acceleration should have fallback to SW codecs
+                {
+                    VideoWriter sw_writer(
+                        filename,
+                        backend,
+                        VideoWriter::fourcc(codecid[0], codecid[1], codecid[2], codecid[3]),
+                        fps,
+                        sz,
+                        {
+                            VIDEOWRITER_PROP_HW_ACCELERATION, VIDEO_ACCELERATION_NONE,
+                        }
+                    );
+                    if (!sw_writer.isOpened()) {
+                        remove(filename.c_str());
+                        throw SkipTestException(backend_name + " VideoWriter on codec " + codecid + " not supported, skipping");
+                    }
+                }
+                remove(filename.c_str());
+                ASSERT_TRUE(hw_writer.isOpened()) << "ANY HW acceleration should have fallback to SW codecs";
+            } else {
+                throw SkipTestException(backend_name + " VideoWriter on " + filename + " not supported with HW acceleration, skipping");
+            }
+        }
+
+        actual_va = static_cast<VideoAccelerationType>(static_cast<int>(hw_writer.get(VIDEOWRITER_PROP_HW_ACCELERATION)));
+        if (va_type != VIDEO_ACCELERATION_ANY && va_type != VIDEO_ACCELERATION_NONE)
+        {
+            ASSERT_EQ((int)actual_va, (int)va_type) << "actual_va=" << actual_va << ", va_type=" << va_type;
+        }
+        std::cout << "VideoWriter " << backend_name << ":" << actual_va << std::endl << std::flush;
+
+        Mat frame(sz, CV_8UC3);
+        for (int i = 0; i < frameNum; ++i) {
+            generateFrame(i, frameNum, frame);
+            if (use_umat) {
+                UMat umat;
+                frame.copyTo(umat);
+                hw_writer.write(umat);
+            }
+            else {
+                hw_writer.write(frame);
+            }
+        }
+    }
+
+    std::ifstream ofile(filename, std::ios::binary);
+    ofile.seekg(0, std::ios::end);
+    int64 fileSize = (int64)ofile.tellg();
+    ASSERT_GT(fileSize, 0);
+    std::cout << "File size: " << fileSize << std::endl;
+
+    // Read video and check PSNR on every frame
+    {
+        VideoCapture reader(
+            filename,
+            CAP_ANY /*backend*/,
+            { CAP_PROP_HW_ACCELERATION, VIDEO_ACCELERATION_NONE }
+        );
+        ASSERT_TRUE(reader.isOpened());
+        double min_psnr = 1000;
+        Mat reference(sz, CV_8UC3);
+        for (int i = 0; i < frameNum; ++i) {
+            Mat actual;
+            if (use_umat) {
+                UMat umat;
+                EXPECT_TRUE(reader.read(umat));
+                umat.copyTo(actual);
+            }
+            else {
+                EXPECT_TRUE(reader.read(actual));
+            }
+            EXPECT_FALSE(actual.empty());
+            generateFrame(i, frameNum, reference);
+            EXPECT_EQ(reference.size(), actual.size());
+            EXPECT_EQ(reference.depth(), actual.depth());
+            EXPECT_EQ(reference.channels(), actual.channels());
+            double psnr = cvtest::PSNR(actual, reference);
+            EXPECT_GE(psnr, psnr_threshold) << " frame " << i;
+            if (psnr < min_psnr)
+                min_psnr = psnr;
+        }
+        Mat actual;
+        EXPECT_FALSE(reader.read(actual));
+        {
+            std::ostringstream ss; ss << actual_va;
+            std::string actual_va_str = ss.str();
+            std::cout << "VideoWriter with acceleration = " << cv::format("%-6s @ %-10s", actual_va_str.c_str(), backend_name.c_str())
+                    << " on codec=" << codecid << " (." << extension << ")"
+                    << ", bitrate = " << fileSize / (frameNum / fps)
+                    << ", with PSNR-original = " << min_psnr
+                    << std::endl << std::flush;
+        }
+        remove(filename.c_str());
+    }
+}
+
+static Ext_Fourcc_PSNR hw_codecs[] = {
+        {"mp4", "MPEG", 29.f, CAP_FFMPEG},
+        {"mp4", "H264", 29.f, CAP_FFMPEG},
+        {"mp4", "HEVC", 29.f, CAP_FFMPEG},
+        {"avi", "MJPG", 29.f, CAP_FFMPEG},
+        {"avi", "XVID", 29.f, CAP_FFMPEG},
+        //{"webm", "VP8", 29.f, CAP_FFMPEG},
+        //{"webm", "VP9", 29.f, CAP_FFMPEG},
+
+        {"mkv", "MPEG", 29.f, CAP_GSTREAMER},
+        {"mkv", "H264", 29.f, CAP_GSTREAMER},
+
+#ifdef _WIN32
+        {"mp4", "MPEG", 29.f, CAP_MSMF},
+        {"mp4", "H264", 29.f, CAP_MSMF},
+#endif
+};
+
+INSTANTIATE_TEST_CASE_P(videoio, videowriter_acceleration, testing::Combine(
+        testing::ValuesIn(hw_codecs),
+        testing::ValuesIn(hw_types),
+        testing::ValuesIn(hw_use_umat)
+));
+
 } // namespace
